@@ -1,7 +1,8 @@
 /*
   Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
   2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
-  2014, 2015 Free Software Foundation, Inc.
+  2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -68,7 +69,7 @@
 # include <netinet/ip.h>
 #endif
 
-#define FTP_NAMES
+#define FTP_NAMES 1
 #include <arpa/ftp.h>
 #include <arpa/inet.h>
 #include <arpa/telnet.h>
@@ -150,6 +151,8 @@ char portstr[8];		/* Numeric port as string.  */
 /* Requester credentials.  */
 struct credentials cred;
 
+jmp_buf errcatch;
+
 static struct sockaddr_storage ctrl_addr;	/* Control address.  */
 static socklen_t ctrl_addrlen;
 static struct sockaddr_storage data_source;	/* Port address.  */
@@ -298,8 +301,8 @@ static struct argp_option options[] = {
   { "timeout", 't', "TIMEOUT", 0,
     "set default idle timeout",
     GRID+1 },
-  { "max-timeout", 'T', NULL, 0,
-    "reset maximum value of timeout allowed",
+  { "max-timeout", 'T', "MAX_TIMEOUT", 0,
+    "set maximum value of timeout allowed",
     GRID+1 },
   { "non-rfc2577", OPT_NONRFC2577, NULL, 0,
     "neglect RFC 2577 by giving info on missing users",
