@@ -1,7 +1,8 @@
 /*
   Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
   2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012,
-  2013, 2014, 2015 Free Software Foundation, Inc.
+  2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -103,8 +104,8 @@ void doit (struct sockaddr *sap, socklen_t salen);
 char *uucico_location = PATH_UUCICO;
 int mypid;
 
-char Username[64];
-char Logname[64];
+char Username[72];
+char Logname[72];
 char *nenv[] = {
   Username,
   Logname,
@@ -276,8 +277,8 @@ doit (struct sockaddr *sap, socklen_t salen)
     }
 
   alarm (0);
-  sprintf (Username, "USER=%s", user);
-  sprintf (Logname, "LOGNAME=%s", user);
+  snprintf (Username, sizeof (Username), "USER=%s", user);
+  snprintf (Logname, sizeof (Logname), "LOGNAME=%s", user);
   dologin (pw, sap, salen);
   setgid (pw->pw_gid);
 #ifdef HAVE_INITGROUPS
@@ -337,13 +338,15 @@ dologout (void)
     }
 }
 
+# define SCPYN(a, b)	strncpy(a, b, sizeof (a) - 1); (a)[sizeof (a) - 1] = 0
+
 /*
  * Record login in wtmp file.
  */
 void
 dologin (struct passwd *pw, struct sockaddr *sap, socklen_t salen)
 {
-  char line[32];
+  char line[NI_MAXHOST]; /* remote is copied here later on */
 #if defined PATH_LASTLOG && defined HAVE_STRUCT_LASTLOG
   int f;
 #endif
@@ -380,7 +383,7 @@ dologin (struct passwd *pw, struct sockaddr *sap, socklen_t salen)
 
   if (hp)
     {
-      strncpy (remotehost, hp->h_name, sizeof (remotehost));
+      SCPYN (remotehost, hp->h_name);
       endhostent ();
     }
   else
@@ -405,9 +408,9 @@ dologin (struct passwd *pw, struct sockaddr *sap, socklen_t salen)
 
     ut.ut_type = USER_PROCESS;
     ut.ut_pid = getpid();
-    strncpy (ut.ut_line, line, sizeof (ut.ut_line));
-    strncpy (ut.ut_user, pw->pw_name, sizeof (ut.ut_user));
-    strncpy (ut.ut_host, remotehost, sizeof (ut.ut_host));
+    SCPYN (ut.ut_line, line);
+    SCPYN (ut.ut_user, pw->pw_name);
+    SCPYN (ut.ut_host, remotehost);
 # ifdef HAVE_STRUCT_UTMPX_UT_SYSLEN
     if (strlen (remotehost) < sizeof (ut.ut_host))
       ut.ut_syslen = strlen (remotehost) + 1;
@@ -425,7 +428,6 @@ dologin (struct passwd *pw, struct sockaddr *sap, socklen_t salen)
 #endif /* HAVE_PUTUTXLINE && !HAVE_LOGWTMPX && !HAVE_LOGWTMP */
 
 #if defined PATH_LASTLOG && defined HAVE_STRUCT_LASTLOG
-# define SCPYN(a, b)	strncpy(a, b, sizeof (a))
   f = open (PATH_LASTLOG, O_RDWR);
   if (f >= 0)
     {
@@ -435,7 +437,7 @@ dologin (struct passwd *pw, struct sockaddr *sap, socklen_t salen)
       time (&t);
       ll.ll_time = t;
       lseek (f, (long) pw->pw_uid * sizeof (struct lastlog), 0);
-      strcpy (line, remotehost);
+      SCPYN (line, remotehost);
       SCPYN (ll.ll_line, line);
       SCPYN (ll.ll_host, remotehost);
       write (f, (char *) &ll, sizeof (ll));
